
# q-expansion of newform 1920.2.b.b, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1920
weight = 2
poly_data = [1, 2, 2, -8, 14, -2, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[-402, 459, 1032, -950, 142, 96, 65, -59], 319], [[-134, -804, 2896, -2018, 260, 32, 128, -126], 319], [[-441, 225, -301, -1504, 251, -28, -112, -129], 319], [[-441, 863, -301, -1504, 251, -28, -112, -129], 319], [[305, 554, 593, -1982, 489, 70, -39, -156], 319], [[-48, 31, 144, -258, 36, 8, 3, -17], 29], [[-307, -1842, 3821, -4590, 629, -60, 79, -322], 319]]

hecke_ring_character_values = [[511, [-1, 0, 0, 0, 0, 0, 0, 0]], [901, [-1, 0, 0, 0, 0, 0, 0, 0]], [641, [-1, 0, 0, 0, 0, 0, 0, 0]], [1537, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0], [0, 0, 1, 0, 0, 0, 0, -1], [0, 1, 0, -1, 1, 0, -1, 0], [0, 1, 0, -1, 0, 0, 0, 1], [0, 0, 0, 0, -1, 0, 1, 1], [2, 0, 0, 0, 1, 1, 1, 0], [2, 1, 0, 1, -1, 1, -1, 0], [-2, -1, 0, -1, 1, -2, 1, 0], [0, 0, 1, 0, 1, 0, -1, -1], [0, -1, -2, 1, 0, 0, 0, 3], [0, 3, 0, -3, 1, 0, -1, -2], [4, 1, 0, 1, 0, -2, 0, 0], [-2, -1, 0, -1, -1, -1, -1, 0], [-2, -2, 0, -2, 0, -2, 0, 0], [0, -1, 2, 1, -1, 0, 1, 2], [0, -1, 0, 1, 1, 0, -1, 2], [4, -1, 0, -1, 0, 2, 0, 0], [4, 2, 0, 2, 2, -2, 2, 0], [2, 0, 0, 0, -2, -2, -2, 0], [0, 2, 3, -2, -1, 0, 1, 1], [0, -1, -2, 1, 0, 0, 0, -2], [0, 2, 0, -2, 0, 0, 0, -2], [2, -2, 0, -2, 0, 2, 0, 0], [2, 3, 0, 3, 1, -2, 1, 0], [0, -2, 3, 2, -2, 0, 2, 1], [0, -1, -2, 1, -2, 0, 2, 6], [0, -3, 0, 3, -3, 0, 3, 4], [0, 4, 2, -4, 1, 0, -1, 3], [0, -4, 1, 4, -2, 0, 2, 3], [0, 3, 2, -3, 1, 0, -1, -6], [0, 2, -6, -2, 1, 0, -1, 1], [2, -4, 0, -4, 1, -3, 1, 0], [-6, 0, 0, 0, -4, 0, -4, 0], [0, 2, 3, -2, 1, 0, -1, 5], [0, 1, 6, -1, 2, 0, -2, -5], [0, -3, 0, -3, 0, 4, 0, 0], [-2, -1, 0, -1, -1, 3, -1, 0], [-2, -8, 0, -8, 0, 0, 0, 0], [0, -1, 2, 1, -1, 0, 1, -2], [0, -4, -2, 4, 0, 0, 0, 0], [0, 6, 0, 6, 0, 4, 0, 0], [-6, -2, 0, -2, 0, -6, 0, 0], [6, 2, 0, 2, 4, -2, 4, 0], [0, -4, 1, 4, -1, 0, 1, -1], [10, 2, 0, 2, -3, 5, -3, 0], [0, 0, 5, 0, 2, 0, -2, -1], [0, -3, -4, 3, -2, 0, 2, 0], [0, 0, 2, 0, 2, 0, -2, -6], [0, -4, -4, 4, -3, 0, 3, 7], [12, 0, 0, 0, -2, -2, -2, 0], [2, 5, 0, 5, -3, 4, -3, 0], [0, 1, 0, -1, 1, 0, -1, 0], [0, 0, 2, 0, 3, 0, -3, -3], [-18, 1, 0, 1, -1, -1, -1, 0], [2, -4, 0, -4, 4, 4, 4, 0], [0, 6, 3, -6, 1, 0, -1, -7], [0, -7, -2, 7, -6, 0, 6, 3], [0, -5, -8, 5, -3, 0, 3, -2], [16, 1, 0, 1, 2, 0, 2, 0], [6, -4, 0, -4, 0, 4, 0, 0], [0, -1, 0, -1, 4, 0, 4, 0], [-12, -6, 0, -6, 2, -6, 2, 0], [2, 4, 0, 4, 4, -8, 4, 0], [-2, 6, 0, 6, 0, 6, 0, 0], [14, 2, 0, 2, 3, -1, 3, 0], [-6, 2, 0, 2, -2, 4, -2, 0], [0, 1, 4, -1, 2, 0, -2, -4], [0, -2, -6, 2, 0, 0, 0, 2], [0, -6, 10, 6, -1, 0, 1, 3], [20, 4, 0, 4, -2, 6, -2, 0], [0, 4, 5, -4, 0, 0, 0, 7], [0, 5, 6, -5, 0, 0, 0, -7], [-10, 0, 0, 0, -1, -1, -1, 0], [-2, 9, 0, 9, 1, -1, 1, 0], [6, 4, 0, 4, 0, 4, 0, 0], [0, 1, -4, -1, 6, 0, -6, -5], [0, -2, 4, 2, 2, 0, -2, -8], [-6, 5, 0, 5, -3, 0, -3, 0], [0, 1, 8, -1, 3, 0, -3, 0], [0, -3, -4, 3, 3, 0, -3, -2], [-4, -6, 0, -6, -2, 10, -2, 0], [10, 6, 0, 6, 0, -6, 0, 0], [0, 4, 9, -4, -1, 0, 1, -5], [0, 5, -4, -5, -2, 0, 2, 0], [0, 1, 4, -1, 1, 0, -1, 0], [10, 6, 0, 6, 2, 4, 2, 0], [14, 9, 0, 9, -1, 2, -1, 0], [0, 6, 3, -6, -4, 0, 4, 5], [0, 7, 2, -7, 4, 0, -4, -2], [12, 4, 0, 4, 2, -6, 2, 0], [0, -4, -7, 4, -2, 0, 2, -1], [0, -5, -6, 5, -3, 0, 3, 2], [6, 0, 0, 0, -1, -1, -1, 0], [-6, 1, 0, 1, -3, 1, -3, 0], [-14, 1, 0, 1, -1, -2, -1, 0], [0, -4, -4, 4, 4, 0, -4, -8], [-4, 9, 0, 9, -6, 2, -6, 0], [0, -6, 2, 6, -2, 0, 2, 0], [8, 3, 0, 3, -6, 0, -6, 0], [6, 2, 0, 2, -6, 0, -6, 0], [0, -1, -6, 1, -2, 0, 2, 6], [0, -1, -12, 1, 3, 0, -3, 0], [-2, -6, 0, -6, -1, 3, -1, 0], [10, 4, 0, 4, 4, 0, 4, 0], [0, -3, -4, 3, 2, 0, -2, -8], [0, -6, -2, 6, -1, 0, 1, 11], [20, -4, 0, -4, 6, 2, 6, 0], [-10, 1, 0, 1, -7, 0, -7, 0], [0, 2, 3, -2, 4, 0, -4, 5], [0, 5, 12, -5, -4, 0, 4, 1], [0, 0, 4, 0, -1, 0, 1, -7], [2, -8, 0, -8, -3, 9, -3, 0], [0, -4, -15, 4, -7, 0, 7, 19], [0, 5, 4, -5, -1, 0, 1, 6], [-4, -3, 0, -3, 2, -6, 2, 0], [10, 11, 0, 11, 5, 5, 5, 0], [22, 0, 0, 0, 6, 2, 6, 0], [0, 3, 14, -3, -5, 0, 5, 2], [0, -9, -8, 9, -3, 0, 3, -2], [2, -6, 0, -6, 0, -2, 0, 0], [-2, 0, 0, 0, -6, -2, -6, 0], [0, -3, -8, 3, 4, 0, -4, -8], [6, -2, 0, -2, 3, -5, 3, 0], [-18, 0, 0, 0, 4, -4, 4, 0], [0, -6, 2, 6, 0, 0, 0, -2], [8, -16, 0, -16, 4, 0, 4, 0], [0, 0, 5, 0, 2, 0, -2, -13], [0, 5, -6, -5, 0, 0, 0, 1], [-18, 4, 0, 4, 3, -5, 3, 0], [-10, -3, 0, -3, 5, -1, 5, 0], [0, 0, 1, 0, -3, 0, 3, -13], [0, 1, 2, -1, 6, 0, -6, -1], [0, 2, 8, -2, -6, 0, 6, -8], [-14, -6, 0, -6, -2, 0, -2, 0], [14, 2, 0, 2, -4, -2, -4, 0], [-8, -1, 0, -1, -4, -8, -4, 0], [-26, -10, 0, -10, 4, -6, 4, 0], [0, 8, 0, -8, 2, 0, -2, -2], [-14, 6, 0, 6, 5, 5, 5, 0], [18, 12, 0, 12, 4, -4, 4, 0], [0, -2, 3, 2, -6, 0, 6, 1], [0, 11, -6, -11, 4, 0, -4, -2], [0, 3, 4, -3, 3, 0, -3, -4], [12, 8, 0, 8, -2, 2, -2, 0], [0, -3, 8, 3, 0, 0, 0, 5], [0, -2, -10, 2, 1, 0, -1, -3], [6, 12, 0, 12, 3, 3, 3, 0], [-30, -3, 0, -3, -3, 1, -3, 0], [0, 1, 6, -1, -2, 0, 2, 7], [0, 7, 4, -7, 1, 0, -1, -6], [-32, 5, 0, 5, -4, -4, -4, 0], [18, 7, 0, 7, -7, 9, -7, 0], [4, -1, 0, -1, 4, 6, 4, 0], [-12, -10, 0, -10, 2, 6, 2, 0], [0, 2, -13, -2, 3, 0, -3, -7], [0, 7, 4, -7, 3, 0, -3, 4], [-14, -12, 0, -12, 2, -2, 2, 0], [10, -9, 0, -9, 1, 2, 1, 0], [0, 5, 0, -5, 8, 0, -8, -4], [0, 8, -2, -8, 1, 0, -1, 7], [0, -2, -9, 2, 6, 0, -6, -3], [0, -3, 0, 3, 1, 0, -1, -4], [0, 2, 0, -2, -3, 0, 3, -3], [-18, -3, 0, -3, 5, 3, 5, 0], [0, 2, -17, -2, 1, 0, -1, -7], [0, 3, -4, -3, 0, 0, 0, -5], [18, -2, 0, -2, -2, 4, -2, 0], [22, 10, 0, 10, 4, -2, 4, 0], [0, -9, 2, 9, 3, 0, -3, -2], [0, -2, 14, 2, -2, 0, 2, 4], [-24, 10, 0, 10, -4, -8, -4, 0], [-6, -6, 0, -6, -8, -2, -8, 0], [0, 2, 3, -2, -5, 0, 5, 5], [0, -4, 4, 4, -10, 0, 10, 10], [-2, -6, 0, -6, -1, 3, -1, 0], [6, -3, 0, -3, -9, -2, -9, 0], [0, 14, -13, -14, 4, 0, -4, -7], [0, 1, 4, -1, -3, 0, 3, 0], [0, -16, 1, 16, -2, 0, 2, 3], [0, 9, 0, -9, 1, 0, -1, 4], [0, -1, 0, 1, 2, 0, -2, 13], [0, 6, 4, -6, 5, 0, -5, 1], [-22, -7, 0, -7, -9, 1, -9, 0], [-6, 1, 0, 1, 3, 2, 3, 0], [0, -1, -6, 1, -4, 0, 4, 15], [16, 1, 0, 1, -2, 0, -2, 0], [10, 1, 0, 1, 5, -4, 5, 0], [-8, 2, 0, 2, 8, 0, 8, 0], [-22, -2, 0, -2, -6, 4, -6, 0], [0, -17, -6, 17, -8, 0, 8, 2], [-46, -2, 0, -2, 1, 1, 1, 0], [-6, 4, 0, 4, -8, 0, -8, 0], [0, 3, -14, -3, -6, 0, 6, 14], [0, -12, 0, 12, -7, 0, 7, 3], [18, -1, 0, -1, -5, -4, -5, 0], [0, -1, -16, 1, 6, 0, -6, 1], [0, 6, 10, -6, 5, 0, -5, 1], [2, 9, 0, 9, 3, 9, 3, 0], [18, 5, 0, 5, -1, -6, -1, 0], [0, 8, 1, -8, 1, 0, -1, -5], [0, 11, -8, -11, 8, 0, -8, -5], [-10, 2, 0, 2, 10, -12, 10, 0], [0, 13, 0, -13, -1, 0, 1, 4], [-26, -8, 0, -8, 0, 8, 0, 0], [0, -10, -13, 10, -5, 0, 5, 1], [0, 5, 4, -5, -2, 0, 2, -12], [0, 8, -12, -8, -2, 0, 2, 6], [-22, -6, 0, -6, -7, 5, -7, 0], [-14, 10, 0, 10, -8, -2, -8, 0], [-30, -16, 0, -16, 4, -8, 4, 0], [0, -8, 1, 8, -10, 0, 10, 7], [0, -9, -2, 9, -8, 0, 8, 10], [24, -12, 0, -12, 4, 0, 4, 0], [6, 15, 0, 15, -1, -4, -1, 0], [0, -12, -3, 12, -12, 0, 12, 15], [0, -18, 4, 18, -6, 0, 6, 8], [-26, -5, 0, -5, 5, -5, 5, 0], [-2, 12, 0, 12, -2, 6, -2, 0], [0, -1, -24, 1, -3, 0, 3, 6], [0, -8, 1, 8, -5, 0, 5, -5], [0, -10, -4, 10, 0, 0, 0, -14], [0, 0, -27, 0, 0, 0, 0, 7], [0, -3, -8, 3, 4, 0, -4, 8], [0, -1, 4, 1, -9, 0, 9, 4], [0, -6, 0, 6, 3, 0, -3, -17], [16, -12, 0, -12, 4, 4, 4, 0], [0, 10, 7, -10, -4, 0, 4, 5], [0, -1, 6, 1, 5, 0, -5, -2], [0, 1, -12, -1, 8, 0, -8, -7], [-30, -8, 0, -8, -11, 5, -11, 0], [0, -2, -5, 2, 9, 0, -9, -3], [0, -1, 24, 1, 1, 0, -1, -2], [0, 13, 0, 13, -8, 4, -8, 0], [2, -5, 0, -5, -9, 1, -9, 0], [34, 11, 0, 11, -5, 12, -5, 0], [-18, 10, 0, 10, 4, 6, 4, 0], [0, -19, 8, 19, -5, 0, 5, 12], [4, -6, 0, -6, 6, -6, 6, 0], [0, -3, -4, 3, 2, 0, -2, -4], [-26, 6, 0, 6, 3, -9, 3, 0], [0, -8, -7, 8, 2, 0, -2, 7], [0, 10, 10, -10, 8, 0, -8, -18], [0, 6, -14, -6, 3, 0, -3, -5], [16, 4, 0, 4, -8, 0, -8, 0], [0, 4, -11, -4, -4, 0, 4, -1], [0, -5, -18, 5, 1, 0, -1, -6], [34, -4, 0, -4, -3, 1, -3, 0], [2, 9, 0, 9, 5, 1, 5, 0], [0, 1, -4, -1, 6, 0, -6, 11], [0, -14, -12, 14, -6, 0, 6, 16], [2, 7, 0, 7, -13, 5, -13, 0], [-6, -9, 0, -9, -1, 12, -1, 0], [-2, -2, 0, -2, -2, 16, -2, 0], [0, -1, 10, 1, -1, 0, 1, -14], [0, 0, -14, 0, 4, 0, -4, -8], [16, -1, 0, -1, 2, -8, 2, 0], [30, 4, 0, 4, -4, 8, -4, 0], [10, 10, 0, 10, -10, -4, -10, 0], [0, 2, 19, -2, -6, 0, 6, -3], [0, -9, -2, 9, -4, 0, 4, -10], [0, 4, 2, -4, -2, 0, 2, -14], [0, -13, -2, 13, -6, 0, 6, -11], [0, -12, -20, 12, -5, 0, 5, 9], [-2, -8, 0, -8, 11, 3, 11, 0], [2, -7, 0, -7, -1, -2, -1, 0], [0, -8, -20, 8, 0, 0, 0, 12], [-12, -15, 0, -15, -6, -10, -6, 0], [6, 14, 0, 14, 2, -12, 2, 0], [0, 12, 2, -12, -4, 0, 4, 4], [-4, 15, 0, 15, -6, 6, -6, 0], [10, 24, 0, 24, -2, 6, -2, 0], [0, 18, 3, -18, 7, 0, -7, -7], [-14, -2, 0, -2, 6, -8, 6, 0], [0, 4, -7, -4, 14, 0, -14, -9], [0, 3, -6, -3, 2, 0, -2, 22], [0, -2, 6, 2, 0, 0, 0, -14], [-22, -7, 0, -7, 9, -12, 9, 0], [0, 7, 14, -7, 9, 0, -9, -22], [2, -3, 0, -3, -1, -11, -1, 0], [0, 14, 11, -14, 1, 0, -1, -7], [14, -1, 0, -1, 3, 3, 3, 0], [0, 10, -6, -10, 6, 0, -6, 0], [40, 19, 0, 19, 6, -8, 6, 0], [24, 2, 0, 2, -4, 8, -4, 0], [10, -6, 0, -6, 8, 6, 8, 0], [-18, 4, 0, 4, -6, -6, -6, 0], [0, 4, 17, -4, 3, 0, -3, -1], [0, -9, -8, 9, -5, 0, 5, 0], [46, -4, 0, -4, -4, 8, -4, 0], [0, -9, -6, 9, -10, 0, 10, 6], [0, 2, 12, -2, 7, 0, -7, -5], [0, -5, -14, 5, 1, 0, -1, -6], [0, -7, -14, 7, 0, 0, 0, -3], [-14, -12, 0, -12, 0, 16, 0, 0], [0, 12, 13, -12, -3, 0, 3, 7], [-2, 2, 0, 2, -2, 4, -2, 0], [0, -19, 4, 19, -5, 0, 5, 16], [32, 7, 0, 7, 6, -4, 6, 0], [-6, -10, 0, -10, -2, 16, -2, 0], [6, -6, 0, -6, -2, 12, -2, 0], [0, 8, -3, -8, 7, 0, -7, -9]]
